"""
OMNeT++ stream and configuration generator
Created: 23/12/2015

@author: Tomas Hegr, CTU in Prague

Example usage:
python confgen.py -i stream13_sim.xml -o ../../omnetpp/EnergoSim/streams/generated/ -ms TR RStations RStation dTS RTUBay RTU -ns 0,1 0,0 0,2 0,2 " -1,-1" 0,1 -mc hostSC -nc 3,3 -sp 2404 -st "normal(19s,2s)" -oi udg_tcp_id11.ini -np ID11
"""
import argparse
from os import sys, path
import logging
from jinja2 import Environment, FileSystemLoader, Template


def pair(arg):
    return [int(x) for x in arg.split(',')]

# Init args
argparser = argparse.ArgumentParser()
argparser.add_argument("-i", "--inputtemplate", help="XML template made for Jinja2", type=str, required=True)
argparser.add_argument("-ii", "--initemplate", help="Omnetpp template for Jinja2", default="TCPUDG.tmpl", type=str)
argparser.add_argument("-id", "--templatedirectory", help="Jinja2 template directory", default="templates", type=str)
argparser.add_argument("-o", "--outputpath", help="Filepath for the script output. There is no file type extension needed.",required=True,type=str)
argparser.add_argument("-oi", "--inifilename", help="Filename ot the omnet ini output.",required=False, default="omnetpp_tcp_udg.ini",type=str)
argparser.add_argument("-np", "--nameprefix", help="Prefix for generated xmls.",required=False, default="", type=str)
argparser.add_argument("-mc", "--modulesClient", help="Space separated hierarchy of client modules.",required=True,type=str,nargs='+')
argparser.add_argument("-nc", "--numberingClient", help="Space separated comma-separated pairs of min and max numbers of client modules. A pair of double -1 means static instance (must be written as \" -1,-1\").",required=True,type=pair,nargs='+')
argparser.add_argument("-ms", "--modulesServer", help="Space separated hierarchy of server modules.",required=True,type=str,nargs='+')
argparser.add_argument("-ns", "--numberingServer", help="pace separated comma-separated pairs of min and max numbers of server modules. A pair of double -1 means static instance (must be written as \" -1,-1\")",required=True,type=pair,nargs='+')
argparser.add_argument("-p", "--port", help="Starting port.",default=15000,type=int)
argparser.add_argument("-st", "--starttime", help="tOpen time.",default="normal(19s,2s)",type=str)
argparser.add_argument("-sp", "--staticport", help="Static server port.",default=-1,type=int)
argparser.add_argument("-op", "--omnetconfpath", help="Omnet conf path.",default="streams/generated/",type=str)
argparser.add_argument("-e", "--exclude", help="Space separated module fullpath names to exclude.",type=str,default=[""],nargs='*')
argparser.add_argument("-l", "--log", help="Log severity.", default="INFO",
                       choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"], type=str)

args = argparser.parse_args()

# Init logging options
logging.basicConfig(level=args.log, format='%(asctime)s %(levelname)s:%(message)s', datefmt='%Y/%m/%d %I:%M:%S %p')
lg = logging.getLogger('omnetppconfgen')

PROFILES_TEMPLATE = """<profiles ver=\"0.1\">
{{ profiles }}
</profiles>"""

def genNames(numbering, modules):
    if len(numbering) > 1:
        if (numbering[:1][0][0] >= 0):
            return ["{}[{}].{}".format(modules[:1][0], i, rest) for i in range(numbering[:1][0][0], numbering[:1][0][1]+1) for rest in genNames(numbering[1:], modules[1:])]
        else:
            return ["{}.{}".format(modules[:1][0], rest) for rest in genNames(numbering[1:], modules[1:])]
    else:
        return ["{}[{}]".format(modules[:1][0], i) for i in range(numbering[:1][0][0], numbering[:1][0][1]+1)]

def generateConfs():
    j2_env = Environment(loader=FileSystemLoader(args.templatedirectory), trim_blocks=True)
    clients = genNames(args.numberingClient,args.modulesClient)
    servers = genNames(args.numberingServer,args.modulesServer)
    port = args.port
    id = 1
    iniconfig = ""
    for client in clients:
        connections = []
        xmls = []
        conf_filename = "{}_{}_{}.xml".format(args.nameprefix,args.inputtemplate[:args.inputtemplate.rfind(".")],client)
        for server in servers:
            if server not in args.exclude and client not in args.exclude:
                if args.staticport > 0:
                    server_port = args.staticport
                else:
                    server_port = args.port

                #Add to connections
                connection = {"conf_filename":path.join(args.omnetconfpath,conf_filename),"server":server,"server_port":server_port, "client":client, "client_port":port,"id":id}
                connections.append(connection)
                lg.debug(connection)


                #Generate XML profile
                con = j2_env.get_template(path.basename(args.inputtemplate)).render(connection=connection)
                xmls.append(j2_env.get_template(path.basename(args.inputtemplate)).render(connection=connection))

                #Increment counters
                port += + 1
                id += 1
        #Generate INI profile
        iniconfig += j2_env.get_template(path.basename(args.initemplate)).render(client=client.replace("[","").replace("]","").replace(".","_"),start_time=args.starttime,connections=connections,name=args.nameprefix)

        #build and save xml
        xml = Template(PROFILES_TEMPLATE).render(profiles="".join(xmls))
        xml_filepath = path.join(args.outputpath,conf_filename)
        try:
            f = open(xml_filepath, "w")
            f.writelines(xml)
            f.close()
            lg.info('The UDG XML has been saved to %s.' % xml_filepath)
        except Exception:
            lg.exception('There was an error during creation of the output files!')

    #save complete ini profile
    iniconfig_filepath = path.join(args.outputpath,args.inifilename)
    try:
        f = open(iniconfig_filepath, "w")
        f.writelines(iniconfig)
        f.close()
        lg.info('The OMNeT++ ini configuration has been saved to %s.' % iniconfig_filepath)
    except Exception:
        lg.exception('There was an error during creation of the output files!')



def main():
    lg.info("Input args: %s" % args)
    if len(args.numberingClient) != len(args.modulesClient):
        lg.error('Size of client modules and numbering are not equal!')
        return
    if len(args.numberingServer) != len(args.modulesServer):
        lg.error('Size of server modules and numbering are not equal!')
        return
    generateConfs()
    lg.info('All OMNeT++ configurations has been generated.')

if __name__ == "__main__":
    sys.exit(main())
